/*

    client.c - NetCrack Official Client 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#ifdef __APPLE__
	#ifdef SAFE
	    #include <unistd.h>
		#include <CoreFoundation/CoreFoundation.h>
		#include <IOKit/IOKitLib.h>
	#endif
#else
#ifndef WIN32
	#ifdef SAFE
	    #include <unistd.h>
		#include <X11/Xlib.h>
		#include <X11/extensions/scrnsaver.h>
	#endif
#endif
#endif
#include "client.h"
#include "socks.h"
#include "pkg.h"
#include "mybrute.h"
#include "crypto/md/md5/md5.h"
#include "crypto/sha/sha-1/sha1.h"

#define FREE(x) {                        \
			if(x) {                      \
				if(x->key) free(x->key); \
				free(x);                 \
				x=NULL;                  \
			}                            \
		}

/********* Shared memory *********/

int verbose    = 0,
    set_daemon = 0,
    death      = 0,
    found      = 0;
    
#ifdef SAFE
int safe       = 0;
#endif
    
char *psw     = NULL,
     *display = NULL;
     
/********* Shared memory *********/

void *thread(void *arg) {
	
#ifdef SAFE
	int seconds = 0;
#endif
	int hlen;
	unsigned char *pass = NULL;
	mybrute_t *brute    = NULL;
	Data *inf           = NULL;
	unsigned char hashin[MAX_HASH_LENGTH],
	              hashout[MAX_HASH_LENGTH];
	
	inf = (Data *)arg;
	
	if(!inf) return NULL;
	
#ifdef SAFE
	if(safe)
		while((seconds = get_idle_time(display)) >= 0 && seconds < SAFE_SECONDS) usleep(500000);
#endif
	
	if(!psw) {
		psw = (char *) malloc(inf->keylen+1);
			
		if(!psw) {
			if(verbose) perror("thread psw malloc");
			found = 2;
			return NULL;
		}	
	}
	
	if(verbose) 
		printf("\nInformation received by the server:\n\n"
		       "\tHash: %s\n"
		       "\tCharset: %s\n"
		       "\tKey: %s\n"
		       "\tKeylen: %d\n"
		       "\tKeyblock: %d\n"
		       "\tHashid: %d\n\n", inf->hash,
		                           inf->cset,
		                           inf->key ? inf->key : "",
		                           inf->keylen,
		                           inf->keyblock,
		                           inf->hashid);
		       
	if((hlen = get_len_by_id(inf->hashid)) < 0) {
		if(verbose) printf("\nUnable to find the hash id.\n\n");
		found = 2;
		return NULL;
	}
	
	string2uchar((unsigned char *)inf->hash, hashin, sizeof(hashin));
	
	brute = mybrute_init(inf->keyblock, (unsigned char *)inf->cset);
	/* The mybrute pass must be initialized at 0 */
	pass = (unsigned char *) calloc(inf->keylen+1, sizeof(unsigned char));
	
	if(!pass) {
		if(verbose) perror("thread pass malloc");
		found = 2;	
		return NULL;
	}
	
	free(brute->buff);
	brute->buff = pass;
	
	if(inf->key) {
		strcpy((char *)brute->buff, inf->key);
		mybrute_restore(brute);
	}
	
	while(!death && mybrute(brute, 0)) {
		
		switch(inf->hashid) {
	
			/* MD5 */
			case 0:
				md5(pass, strlen((char *)pass), hashout);
			break;
				
			/* SHA-1 */
			case 1:
				sha1(pass, strlen((char *)pass), hashout);
			break;
				
			default:
				memset(hashout, 0, sizeof(hashout));
			break;
				
		}
		
		if(!memcmp(hashout, hashin, hlen)) {
			found = 1;	
			break;
		}
		
    }
    
    if(found == 1)
		mystrcpy(psw, (char *)pass, inf->keylen+1);
    else {
    	found = -1;
    	free(pass);
    }
    
    return NULL;
	
}

int do_auth(int sock, char *password) {
	
	int i,
	    error = 0;
	char *ret = NULL;
	
	if(!password) return -1;
	
	send_ar_packet(sock, password, strlen(password));
	
	for(i=0; i<3; i++) {
		
		ret = socket_read_pkg(sock, &error);
		
		if(error && verbose) {
			printf("\nConnection closed by the host.\n\n");
			break;
		}
		
		if(ret && !strcmp(ret, PKG_AG)) return 0;
		
	}
	 
	return -1;
	
}

void listen_from_server(int sock) {

	int error = 0;
	char *ret = NULL;
	Data *information = NULL;
	pthread_t td;
	
	while(1) {
		
		ret = socket_read_pkg(sock, &error);
		
		if(error) {
			if(verbose) printf("\nConnection closed by the host.\n\n");
			break;
		}
		
		/* CS */
		if(found == 1) {
			send_cs_packet(sock, psw);
			FREE(information);
			break;
		}
		
		/* CF */
		if(found < 0) {
			send_simple_packet(sock, PKG_CF);
			found = 0;
		}
		
		/* Thread error */
		if(found == 2) {
			send_simple_packet(sock, PKG_BYE);
			FREE(information);
			break;
		}

		if(!ret) continue;
		
		/* PING */
		if(!strcmp(ret, PKG_PING)) {
			send_simple_packet(sock, PKG_PONG);
			continue;
		}
		
		/* CI */
		if(!strncmp(ret, PKG_CI, 2)) {
			information = get_data_from_ci(ret);
			if(!information) continue;
			send_simple_packet(sock, PKG_IR);
			continue;
		}
		
		/* DN */
		if(!strcmp(ret, PKG_DN)) {
			information->key = NULL;
			pthread_create(&td, NULL, thread, (void *)information);
			continue;
		}
		
		/* DT */
		if(!strncmp(ret, PKG_DT, 2) && strlen(ret) > 3) {
		
			information->key = (char *) realloc(information->key, strlen(ret)-2);
			
			if(!information->key) {
				FREE(information);
				send_simple_packet(sock, PKG_BYE);
				break;
			}
			
			mystrcpy(information->key, ret+3, strlen(ret+3));
			delimit_str(information->key);
				
			pthread_create(&td, NULL, thread, (void *)information);
			continue;
		
		}
		
		/* PF */
		if(!strcmp(ret, PKG_PF)) {
			death = 1;
			pthread_join(td, NULL);
			FREE(information);
			if(verbose) printf("\nPassword found by other host.\n\n");
			break;
		}
					
		/* BYE */
		if(!strcmp(ret, PKG_BYE)) {
			death = 1;
			pthread_join(td, NULL);
			FREE(information);
			if(verbose) printf("\nSession complete.\n\n");
			break;
		}
		
	}
	
}

char *split(char *in, char *delone, char *deltwo) {
	
	int len;
	char *start  = NULL,
	     *end    = NULL,
	     *buffer = NULL;

	if(!(start = strstr(in, delone))) {
		if(!set_daemon && verbose) printf("\nUnable to find the occourence of: %s\n\n", delone);
		return NULL;
	}
	
	start += strlen(delone);

	if(!(end = strstr(start, deltwo))) {
		if(!set_daemon && verbose) printf("\nUnable to find the occourence of: %s\n\n", deltwo);
		return NULL;
	}
		
	len = end-start+1;
	
	if(!(buffer = (char *) calloc(len, sizeof(char)))) {
		if(!set_daemon && verbose) perror("Unable to allocate dynamic memory");
		return NULL;
	}
	
	memcpy(buffer, start, len-1);
	buffer[len]=0;
	
	return buffer;
	
}

Data *get_data_from_ci(char *pkg) {
	
	int i,
	    b64csetlen;
	char *string = NULL,
	     *tmp    = NULL;
	Data *information = NULL;
	
	if(!pkg || strlen(pkg) < 3) return NULL;
	
	information = (Data *) malloc(sizeof(Data));
	
	if(!information) {
		if(verbose) perror("get_data_from_ci information malloc");
		return NULL;
	}
	
	information->key = NULL;
	
	information->cset = split(pkg, " ", " ");
	if(!information->cset) {
		free(information);
		information = NULL;
		return NULL;
	}
	
	information->hash = split(pkg+3+strlen(information->cset), " ", " ");
	if(!information->hash) {
		free(information);
		information = NULL;
		return NULL;
	}
	
	tmp = split(pkg+3+strlen(information->cset)+strlen(information->hash), " ", "\n");
	if(!tmp) {
		if(information) {
			free(information);
			information = NULL;
		}
		return NULL;
	}
	
	string = strtok(tmp, "-");
	
	for(i=0; i<3 && string; i++) {
		
		if(!i) {
			information->keylen = (unsigned int) atoi(string);	
		}
		else {
			if(i == 1) {
				information->keyblock = (unsigned int) atoi(string);
			}
			else
				information->hashid = (unsigned int) atoi(string);
		}
		
		string = strtok(NULL, "-");
		
	}
	
	if(i<3) {
		if(information) {
			free(information);
			information = NULL;
		}
		return NULL;
	}
	
	b64csetlen = strlen(information->cset);
	information->cset = (char *)base64_decode((unsigned char *)information->cset, &b64csetlen);
	
	if(!information->cset) {
		free(information);
		information = NULL;
		return NULL;
	}
	
	return information;
	
}

/* Base64 decoder by Luigi Auriemma */
unsigned char *base64_decode(unsigned char *data, int *size) {
    int     len,
            xlen,
            a   = 0,
            b   = 0,
            c,
            step;
    unsigned char      *buff,
            *limit,
            *p;
    static const unsigned char base[128] = {   // supports also the Gamespy base64
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x00,0x3e,0x00,0x3e,0x00,0x3f,
        0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,0x3c,0x3d,0x00,0x00,0x00,0x00,0x00,0x00,
        0x00,0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,
        0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x3e,0x00,0x3f,0x00,0x3f,
        0x00,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,
        0x29,0x2a,0x2b,0x2c,0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x00,0x00,0x00,0x00,0x00
    };

    if(!size || (*size < 0)) {      // use size -1 for auto size!
        len = strlen((char *)data);
    } else {
        len = *size;
    }
    xlen = ((len >> 2) * 3) + 1;    // NULL included in output for text
    buff = malloc(xlen);
    if(!buff) return(NULL);

    p = buff;
    limit = data + len;

    for(step = 0; /* data < limit */; step++) {
        do {
            if(data >= limit) {
                c = 0;
                break;
            }
            c = *data;
            data++;
            if((c == '=') || (c == '_')) {  // supports also the Gamespy base64
                c = 0;
                break;
            }
        } while(c && ((c <= ' ') || (c > 0x7f)));
        if(!c) break;

        switch(step & 3) {
            case 0: {
                a = base[c];
                break;
            }
            case 1: {
                b = base[c];
                *p++ = (a << 2)        | (b >> 4);
                break;
            }
            case 2: {
                a = base[c];
                *p++ = ((b & 15) << 4) | (a >> 2);
                break;
            }
            case 3: {
                *p++ = ((a & 3) << 6)  | base[c];
                break;
            }
        }
    }
    *p = 0;

    len = p - buff;
    if(size) *size = len;
    if((len + 1) != xlen) {             // save memory
        buff = realloc(buff, len + 1);  // NULL included!
        if(!buff) return(NULL);
    }
    return(buff);
}

int string2uchar(unsigned char *in, unsigned char *md5_out, size_t outlen) {

    int i,
        t;
    unsigned char *p;

    p = in;
    
    for(i=0; i<outlen; i++) {
        if(sscanf((char *)p, "%02x", &t) != 1) return -1;
        md5_out[i] = t;
        p += 2;
    }
    
    return 0;

}

#ifdef SAFE

int get_idle_time(char *display) {
	
#ifdef __APPLE__

	static io_service_t macIOsrvc;
	CFTypeRef property;
	uint64_t idle = 0;
	
#else

	Display *disp = NULL;
    XScreenSaverInfo *screeninf = NULL;
    int xss_event_basep,
        xss_error_basep,
        idle;
    
#endif

#ifdef __APPLE__

	if(!macIOsrvc) {
		mach_port_t master;
		IOMasterPort(MACH_PORT_NULL, &master);
		macIOsrvc = IOServiceGetMatchingService(master, IOServiceMatching("IOHIDSystem"));
	}

	property = IORegistryEntryCreateCFProperty(macIOsrvc, CFSTR("HIDIdleTime"), kCFAllocatorDefault, 0);
	
	CFNumberGetValue((CFNumberRef)property, kCFNumberSInt64Type, &idle);
	
	CFRelease(property);

	/* Time in seconds */
	return idle/1000000000;

#else

    if(!display) display = getenv("DISPLAY");

    if(!(disp = XOpenDisplay(display))) {
        if(verbose && !set_daemon) printf("\nCouldn't open X display %s\n\n", display);
        exit(-1);
    }
  
    if(!XScreenSaverQueryExtension(disp, &xss_event_basep, &xss_error_basep)) {
        if(verbose && !set_daemon) printf("\nX Screen Saver extension is not available!\n\n");
        XCloseDisplay(disp);
        exit(-1);
    }
      
    if(!(screeninf = XScreenSaverAllocInfo())) {
        if(verbose && !set_daemon) printf("\nFailed to allocate X Screen Saver information memory!\n\n");
        XCloseDisplay(disp);
        exit(-1);
    }

    if(!XScreenSaverQueryInfo(disp, DefaultRootWindow(disp), screeninf)) {
         if(verbose && !set_daemon) printf("\nFailed to get X Screen Saver information!\n\n");
         XCloseDisplay(disp);
         exit(-1);
     }
    
    /* Time in seconds */
    idle = screeninf->idle/1000;

    XCloseDisplay(disp);

    return idle;
    
#endif

}

#endif

int get_len_by_id(int id) {
	
	switch(id) {
	
		/* MD5 */
		case 0:
			return 16;
		break;
		
		/* SHA-1 */	
		case 1:
			return 20;
		break;
			
		default:
			return -1;
		break;
					
	}
	
	return -1;
	
}

int delimit_str(char *str) {
	
	char *p;
	
	for(p=str; *p && *p != '\n' && *p != '\r'; p++);
	
	*p = 0x00;
	
	return p-str;
	
}

int mystrcpy(char *out, char *in, size_t outlen) {
	
	int i;
	
	if(outlen <= 0) return -1;
	
	for(i=0, outlen--; i<outlen && in[i]; i++) out[i] = in[i];
	
	out[i] = 0;
	
	return i;
	
}
