/*

    options.c - NetCrack Server 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#ifndef WIN32
	#include <unistd.h>
#endif
#ifdef SQL
	#include <mysql/mysql.h>
#endif
#include "options.h"
#include "server.h"

void usage(char *cmd) {

	printf("\nNetCrack Server %s - (C) 2009 Salvatore Fresta"
               "\nhttp://www.salvatorefresta.net"
               "\n"
               "\nusage: %s [options]\n"
               "\n"
 	           "Required options:\n"
               "\n"
               "\t--port    <value>  : TCP port to listen on\n"
#ifndef SQL
               "\t--hashid  <value>  : Hash Identifier\n"
               "\t--keylen  <value>  : Max key length\n"
               "\t--hash    <value>  : A supported hash function to crack\n"
               "\n"
               "Charset options [default azAZ09]:\n"
               "\n"
               "\t--c1               : Add alpha lowercase charset [a-z]\n"
               "\t--c1               : Add alpha uppercase charset [A-Z]\n"
               "\t--c3               : Add numeric charset [0-9]\n"
               "\t--c4               : Add special charset [Space-~]\n"
#endif
               "\n"
               "Common options:\n"
               "\n"
#ifdef SQL
               "\t--verbose          : Enable verbose mode\n"
#ifndef WIN32
               "\t--daemon           : Start as daemon\n"
#endif
#endif
               "\t--list             : Hash IDs list\n"
               "\t--help             : Display this help\n"
               "\n",
		       VERSION, cmd);

}

#ifdef SQL

MYSQL *do_mysql_connection(int verbose) {
	
	MYSQL *dbsock;
	
	dbsock = mysql_init(NULL);

	if(!mysql_real_connect(dbsock, MYSQL_SERVER, MYSQL_USER, MYSQL_PSW, MYSQL_DB, 0, NULL, 0)) {
		if(verbose) fprintf(stderr, "\nmysql_real_connect: %s\n\n", mysql_error(dbsock));
		return NULL;
	}
	
	return dbsock;
	
}

void do_mysql_close(MYSQL *dbsock) {
	
	mysql_close(dbsock);
	
}

Analys * get_new_hash_from_mysql(MYSQL *dbsock, int verbose) {

	Analys *data = NULL;
	MYSQL_RES *res = NULL;
	MYSQL_ROW row;
	int hlen;
	char *query = NULL;
	
	vspr(&query, "SELECT hash, charset, keylen, hashid FROM %s WHERE processed = 0 ORDER BY date", MYSQL_TABLE);

	if(mysql_query(dbsock, query)) {
		if(verbose) fprintf(stderr, "\n%s\n\n", mysql_error(dbsock));
		free(query);
		return NULL;
	}

	if(!(res = mysql_store_result(dbsock))) {
		if(verbose) fprintf(stderr, "\n%s\n\n", mysql_error(dbsock));
		free(query);
		return NULL;
	}

	if((long)mysql_num_rows(res) <= 0) {
		mysql_free_result(res);
		free(query);
		return NULL;
	}

	row = mysql_fetch_row(res);

	if(!row) {
		if(verbose) fprintf(stderr, "\n%s\n\n", mysql_error(dbsock));
		mysql_free_result(res);
		free(query);
		return NULL;
	}
	
	hlen = get_len_by_id(atoi(row[3]))*2;

	if(strlen(row[0]) != hlen || check_hash_string(row[0], hlen)) {
		if(verbose) printf("\nError: please set a valid hash.\n\n");
		mysql_free_result(res);
		free(query);
		exit(-1);
	}

	if(atoi(row[2]) <= 0) {
		if(verbose) printf("\nError: please set a valid key length.\n\n");
		mysql_free_result(res);
		free(query);
		exit(-1);
	}

	if(atoi(row[3]) < 0 || atoi(row[3]) > 1) {
		if(verbose) printf("\nError: please set a valid hash id.\n\n");
		mysql_free_result(res);
		free(query);
		exit(-1);
	}

	data = (Analys *) calloc(1, sizeof(Analys));

	if(!data) {
		if(verbose) perror("\nget_new_hash_from_mysql data calloc");
		mysql_free_result(res);
		free(query);
		return NULL;
	}

	strncpy(data->hash, row[0], MAX_HASH_LENGTH*2+1);
	strncpy(data->charset, row[1], 16);
	data->hashid = (unsigned int) atoi(row[3]);
	data->keylen = (unsigned int) atoi(row[2]);
	
	if(data->keylen <=  5 && data->keylen > 1)
		data->keyblock = data->keylen-1;
	else
		if(data->keylen == 1)
			data->keyblock = 0;
		else
			data->keyblock = 5;

	mysql_free_result(res);
	free(query);
	
	return data;

}

int update_hash_on_mysql(MYSQL *dbsock, char *hash, char *psw, int found, int verbose) {
	
	char *query = NULL,
         *pswescape = NULL;

	if(psw) {
		pswescape = calloc(strlen(psw)*2+1, sizeof(char));
		if(!pswescape) {
			if(verbose) perror("update_hash_on_mysql pswescape calloc");
			return -1;
		}
		/* Avoid SQL Injection */
		mysql_real_escape_string(dbsock, pswescape, psw, strlen(psw));
	}
	
	if(found)
		vspr(&query, "UPDATE %s SET password = '%s', found = 1, processed = 1 WHERE hash = '%s'", MYSQL_TABLE, pswescape, hash);
	else
		vspr(&query, "UPDATE %s SET found = 0, processed = 1 WHERE hash = '%s'", MYSQL_TABLE, hash);
	
	if(mysql_query(dbsock, query)) {
		if(verbose) fprintf(stderr, "\n%s", mysql_error(dbsock));
		if(pswescape) free(pswescape);
		free(query);
		return -1;
	}
	
	if(query) free(query);
	
	return 0;
	
}

#endif

void get_hash_id_list() {

	printf("\n\tHash - ID\n"
               "\n\tMD5  - 0"
               "\n\tSHA1 - 1"
               "\n\n");

}

#ifndef WIN32

int daemonize(void) {
        
	pid_t pid;
        
    if((pid=fork()) < 0)
		return 0;
    else
        if(pid > 0) exit(0);
        
    setsid();
                
    return 1;
        
}

#endif

int check_hash_string(char *in, int inlen) {

	int i;

	if(inlen % 8 != 0) return -1;

	for(i=0; i<inlen; i++)
		if(!((in[i] >= 65 && in[i] <= 70) || (in[i] >= 97 && in[i] <= 102) || (in[i] >= 40 && in[i] <= 57))) return -1;

	return 0;

}

int vspr(char **buff, char *fmt, ...) {
	
    va_list ap;
    int     len,
            mlen;
    char    *ret = NULL;

    mlen = strlen(fmt) + 128;
    va_start(ap, fmt);
    
    while(1) {
    	
        ret = (char *) malloc(mlen);
        if(!ret) return -1;
        
        len = vsnprintf(ret, mlen, fmt, ap);
        
        if((len >= 0) && (len < mlen)) break;
        
        mlen = len < 0 ? mlen+128 : len+1;
        
        free(ret);
        
    }
    
    va_end(ap);

    *buff = ret;
    
    return len;
    
}

void print_connected_client(int num) {
	
	if(num <= 0)
		printf("\rConnected clients: 0");
	else
		printf("\rConnected clients: %d", num);
	fflush(stdout);
	
}

void print_startend_timestamp(time_t start, time_t end) {
	
    struct tm  *ts;
    char buf[80];
 
    ts = localtime(&start);
    strftime(buf, sizeof(buf), "%a %Y-%m-%d %H:%M:%S %Z", ts);
    printf("\n\nStart date : %s\n", buf);
    
    ts = localtime(&end);
    strftime(buf, sizeof(buf), "%a %Y-%m-%d %H:%M:%S %Z", ts);
    printf("End date   : %s\n\n", buf);

}
