/*

    server.c - NetCrack Server 1.0

    Copyright 2009 Salvatore Fresta aka drosophila

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA

    http://www.gnu.org/licenses/gpl-2.0.txt

*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#ifdef WIN32
	#include <winsock.h>
	#define sleep       Sleep
    #define waitms      sleep
	typedef int socklen_t;
	#define ASYNC       int async = 1; \
                        if(ioctlsocket(sockd, FIONBIO, (void *)&async) < 0)
#else
	#include <arpa/inet.h>
	#include <sys/types.h>
	#include <sys/socket.h>
	#include <netinet/in.h>
	#include <unistd.h>
	#include <netdb.h>
	#define waitms(x)   usleep(x * 1000);
	#define ASYNC       int async; \
                        if((async = fcntl (sockd, F_GETFL)) < 0 || \
                          fcntl(sockd, F_SETFL, async | O_NONBLOCK) < 0)
#endif
#include <pthread.h>
#include <fcntl.h>
#ifdef SQL
	#include <mysql/mysql.h>
#endif
#include "options.h"
#include "pkg.h"
#include "socks.h"
#include "server.h"
#include "mybrute.h"
#include "crypto/md/md5/md5.h"
#include "crypto/sha/sha-1/sha1.h"

/********* Shared memory *********/

pthread_mutex_t mutex = PTHREAD_MUTEX_INITIALIZER; /* Mutex */

Analys *info_data = NULL;   /* Information about cracking */
Client *cl_list   = NULL;   /* Connected clients list */
Priority *plist   = NULL;   /* Keys priority list */

char ip[16],      /* Cracker's IP address */
     *psw = NULL; /* Password found */

int pswfound   = 0,  /* Flag password found */
    t_death    = -1, /* Clients dead counter */
    c_session  = 0,  /* Cracking session flag */
    plist_only = 0;  /* Read from priority list only */

/********* Shared memory *********/

int start_server(int port, int clients, int verbose) {

#ifdef SQL	
	MYSQL *dbsock = NULL;
#endif

	struct sockaddr_in server,
	                   client;
	int sockd,
	    sock_cl,
	    t_alive = 0,
	    i;
	    
	time_t tstart = 0,
	       tend = 0;
	
	socklen_t len_cl;
	pthread_t td;
	
#ifdef WIN32
    WSADATA wsadata;
    if(WSAStartup(MAKEWORD(1,0), &wsadata)) return -1;
#endif
	
	sockd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

	if(sockd <= 0) {
#ifdef WIN32
		if(verbose) printf("\nstart_server socket: unable to open a socket.");
#else
		if(verbose) perror("\nstart_server socket");
#endif
		return -1;
	}
	
	memset((void *)&server, 0, sizeof(server));
	
	server.sin_family = AF_INET;
	server.sin_addr.s_addr = htonl(INADDR_ANY);
	server.sin_port = htons(port);
	
	if(bind(sockd, (struct sockaddr *)&server, sizeof(server)) < 0) {
#ifdef WIN32
		if(verbose) printf("\nstart_server bind: unable to bind.");
#else
		if(verbose) perror("\nstart_server bind");
#endif
		socket_close(sockd);
		return -1;
	}
	
	if(listen(sockd, clients) < 0) {
#ifdef WIN32
		if(verbose) printf("\nstart_server listen: unable to listen.");
#else
		if(verbose) perror("\nstart_server listen");
#endif
		socket_close(sockd);
		return -1;
	}

	ASYNC {
#ifdef WIN32
		if(verbose) printf("start_server async: unable to set the asynchronous mode.\n");
#else
		if(verbose) perror("start_server async");
#endif
		socket_close(sockd);
		return -1;
	}

#ifdef SQL
	dbsock = do_mysql_connection(1);
	
	if(!dbsock) {
		socket_close(sockd);
		return -1;
	}
#else
	if(info_data) c_session = 1;
#endif

#ifndef SQL
	if(verbose) printf("\nListening...\n\n----->\n\n");
#endif

	while(1) {

#ifdef SQL		
		if(!info_data) {
			
			info_data = get_new_hash_from_mysql(dbsock, verbose);
			
			if(!info_data) continue;
			
			pswfound = 0;
			
			gennewkey(0, 0, NULL, 1);
			
			c_session = 1;
			
			t_alive = 0;
			t_death = -1;
			
			if(verbose) printf("\nListening...\n\n----->\n\n");
			
		}
#endif		
		if(!psw) {
			psw = (char *) calloc(info_data->keylen+1, sizeof(char));
			
			if(!psw) {
				if(verbose) perror("start_server psw calloc");
				continue;
			}
		}
		
		/* Password found */
		if(pswfound) {
			
			tend = time(NULL);
			
			/* Wait the clients disconnection */
			while(t_death != t_alive) waitms(500);
			
			if(verbose) {
				printf("\n\nPassword found by %s\n\n"
						"ASCII rappresentation : %s\n"
						"Hex   rappresentation : ", ip, psw);
				if(!strlen(psw))
					printf("00");
				else 
					for(i=0; i<strlen(psw); i++) printf("%02x", psw[i]);
				print_startend_timestamp(tstart, tend);
			}
#ifdef SQL			
			update_hash_on_mysql(dbsock, info_data->hash, psw, 1, verbose);
#endif			

			free(psw);
			free(info_data);
			psw = NULL;
			info_data = NULL;

#ifdef SQL
			if(verbose) printf("Session complete.\n\n<-----\n\n");
			continue;
#else			
			break;
#endif
			
		}
		
		/* Password not found */ 
		if(!pswfound && t_death == t_alive && !c_session) {
			
			tend = time(NULL);
			
			if(verbose) {
				printf("\n\nPassword not found!\n\n");
				print_startend_timestamp(tstart, tend);
			}

#ifdef SQL
			update_hash_on_mysql(dbsock, info_data->hash, psw, 0, verbose);
#endif

			free(psw);
			free(info_data);
			psw = NULL;
			info_data = NULL;

#ifdef SQL
			if(verbose) printf("Session complete.\n\n<-----\n\n");
			continue;
#else			
			break;
#endif			

		}
				
		if(c_session) {
			
			if(!t_alive && !test_null_hash()) {
				t_death = 0;
				continue;
			}

			len_cl = sizeof(client);
			
			if((sock_cl = accept(sockd, (struct sockaddr *)&client, &len_cl)) >= 0) {
				cl_list = add_client(cl_list, sock_cl, inet_ntoa((struct in_addr)client.sin_addr));
				
				pthread_create(&td, NULL, thread,(void *)cl_list);
				
				if(t_death < 0) {
					tstart = time(NULL);
					t_death = 0;
				}
				t_alive++;
			}
			
			if(verbose) {
				if(t_death < 0)
					print_connected_client(0);
				else
					print_connected_client(t_alive-t_death);
			}
		
		}
		
		waitms(500);

	}
	
#ifdef SQL
	do_mysql_close(dbsock);
#endif

	socket_close(sockd);
	
	return 0;
	
}

void *thread(void *arg) {

	int sock;
	Client *record = NULL;
	
	record = (Client *)arg;
	
	if(!record) {
		pthread_mutex_lock(&mutex);
		t_death++;
		pthread_mutex_unlock(&mutex);
		return NULL;
	}
	
	sock = record->sock;
	
	/* Authentication successful */
	if(check_auth(sock) == 0) listen_from_client(sock, record->ip);

	socket_close(sock);
	
	pthread_mutex_lock(&mutex);
	cl_list = del_client(cl_list, sock);
	t_death++;
	pthread_mutex_unlock(&mutex);
	
	return NULL;
	
}

void listen_from_client(int sock, char *myip) {
	
	time_t t1, t2;    /* Time for PING and CI packets */
	int error = 0,    /* Client connection error */
	    infsent = 0,  /* Flag for CI packets */
	    infrec = 0,   /* Flag for IR packets */
	    pingsent = 0, /* PING packets sent */
	    pingrec = 0,  /* PONG packets received */
	    datasent = 0, /* DT or DN packets sent */
	    dataresp = 0, /* CF packets received */
	    hlen;         /* Hash length */
	char *ret = NULL, /* Pointer to received packet */
         *key = NULL; /* Pointer to key to crack */
	unsigned char hashin[MAX_HASH_LENGTH],
	              hashout[MAX_HASH_LENGTH];
	
	t1 = t2 = time(NULL);

	while(1) {
		
		ret = socket_read_spkg(sock, &error);
		
		/* Connection error */
		if(error) {
			if(key) {
	  			/* Insert the sent key into the priority list */
				pthread_mutex_lock(&mutex);
				plist = add_priority(plist, key);
				pthread_mutex_unlock(&mutex);
				/* Read priority list only if the bruteforce is ended */
				if(!c_session) {
					plist_only = 1;
					c_session = 1;
				}
			}	
			break;
		}
		
		/* PING */
		if(time(NULL)-t1 >= SECONDS_TO_PING && infrec) {
			if(pingsent++ != pingrec) {
				send_simple_packet(sock, PKG_BYE);
				if(key) {
					pthread_mutex_lock(&mutex);
					plist = add_priority(plist, key);
					pthread_mutex_unlock(&mutex);
					if(!c_session) {
						plist_only = 1;
						c_session = 1;
					}
				}	
				break;
			}
			send_simple_packet(sock, PKG_PING);
			t1 = time(NULL);
		}
		
		/* PF */
		if(pswfound) {
			send_simple_packet(sock, PKG_PF);
			break;
		}
		
		/* CI */
		if(!infsent) {
			
			send_ci_packet(sock, info_data->hash, info_data->charset, info_data->keylen, info_data->keyblock, info_data->hashid);		

			infsent = 1;
			continue;
			
		}
		
		/* IR timeout */
		if(time(NULL)-t2 >= SECONDS_TO_INFREC && !infrec) {
			if(infsent == 3) {
				send_simple_packet(sock, PKG_BYE);
				break;
			}
			send_ci_packet(sock, info_data->hash, info_data->charset, info_data->keylen, info_data->keyblock, info_data->hashid);
			++infsent;
			t2 = time(NULL);
		}

		/* DT */
		if(datasent == dataresp && infsent && infrec) {
			
			key = (char *) gennewkey(info_data->keylen,
			                        info_data->keyblock,
			                        (unsigned char *)info_data->charset,
			                        0);
			                            
			/* Bruteforce ended */
			if(!key) {
				send_simple_packet(sock, PKG_BYE);
				break;
			}
			
			if(!key[0])
				send_simple_packet(sock, PKG_DN);
			else
				send_data_packet(sock, key);
				
			datasent++;
			
		}
		
		/* No packet received */
		if(!ret) continue;
		
		/* IR */
		if(!strcmp(ret, PKG_IR)) {
			infrec = 1;
			free(ret);
			ret = NULL;
			continue;
		}
		
		/* PONG */
		if(!strcmp(ret, PKG_PONG) && infrec) {
			pingrec++;
			free(ret);
			ret = NULL;
			continue;
		}
		
		/* CS */
		if(!strncmp(ret, PKG_CS, 2) && infrec && strlen(ret) > 3) {
			
			pthread_mutex_lock(&mutex);
			
			mystrcpy(psw, ret+3, info_data->keylen+1);
			delimit_str(psw);
			
			hlen = get_len_by_id(info_data->hashid);

			string2uchar((unsigned char *)info_data->hash, hashin, sizeof(hashin));

			switch(info_data->hashid) {

				/* MD5 */
				case 0:
					md5((unsigned char *)psw, strlen(psw), hashout);
				break;

				/* SHA-1 */
				case 1:
					sha1((unsigned char *)psw, strlen(psw), hashout);
				break;
				
				default:
					memset(hashout, 0, sizeof(hashout));
				break;

			}

			/* Incorrect received password */
			if(memcmp(hashout, hashin, hlen))				
				plist = add_priority(plist, key);
			else {
				mystrcpy(ip, myip, sizeof(ip));
				pswfound = 1;
			}
			
			pthread_mutex_unlock(&mutex);
			
			send_simple_packet(sock, PKG_BYE);
			
			break;
			
		}
		
		/* CF */
		if(!strcmp(ret, PKG_CF) && infrec) {
			dataresp++;
			free(ret);
			ret = NULL;
			continue;
		}
		
		/* BYE */
		if(!strcmp(ret, PKG_BYE) && infrec) {
			if(datasent != dataresp) {
				if(key) {
					pthread_mutex_lock(&mutex);
					plist = add_priority(plist, key);
					pthread_mutex_unlock(&mutex);
					if(!c_session) {
						plist_only = 1;
						c_session = 1;
					}
				}
			}
			break;
		}
		
	}
	
	if(ret) {
		free(ret);
		ret = NULL;
	}
	
	if(key && key[0]) free(key);
	
}

int check_auth(int sock) {
	
	int i,
	    error = 0;
	char *ret = NULL,
	     *pass = NULL;
	
	for(i=0; i<3; i++) {
			
		ret = socket_read_spkg(sock, &error);
		
		if(error) break;

		if(!ret) continue;
		
		if(!strncmp(ret, PKG_AR, 2) && strlen(ret) > 3) {
				
			pass = ret+3;
			delimit_str(pass);
			
			if(pass && !strcmp(pass, PASSWORD)) {
				send_simple_packet(sock, PKG_AG);
				break;
			}
			else
				send_simple_packet(sock, PKG_AD);
			
		}
		
	}
	
	return (i < 3 && !error) ? 0 : -1;
	
}

Client *add_client(Client *list, int sock, char *ip) {

	Client *tmp = NULL;
	
	tmp = (Client *) calloc(1, sizeof(Client));
	if(!tmp) return NULL;

	tmp->sock = sock;
	mystrcpy(tmp->ip, ip, 16);
	
	tmp->next = list;

	return tmp;
	
}

Client *del_client(Client *list, int sock) {
	
	Client *punb = NULL,
		   *punn = NULL;
		   
	if(!list) return NULL;
	
	punn = list;
	
	if(punn->sock == sock) {
		punb = punn->next;
		free(punn);
		return punb;
	}
	
	punb = punn;
	punn = punn->next;
	
	while(punn) {
		
		if(punn->sock == sock) {
			punb->next = punn->next;
			free(punn);
			break;
		}
		
		punb = punn;
		punn = punn->next;
		
	}

	return list;
	
}

Priority *add_priority(Priority *plist, char *pkey) {
	
	Priority *tmp = NULL,
	         *p   = NULL;
	
	if(plist) {
		p = plist;
		
		while(p->next != NULL) p = p->next;
	}
		
		tmp = (Priority *) calloc(1, sizeof(Priority));
		
		if(!tmp) {
			perror("\nadd_priority tmp calloc");
			return NULL;
		}
		
		tmp->key = (char *) calloc(strlen(pkey), sizeof(char));
		
		if(!tmp->key) {
			perror("\nadd_priority tmp->key calloc");
			free(tmp);
			return plist;
		}
		
		strcpy(tmp->key, pkey);
		tmp->next = NULL;
		
	if(!plist)	
		plist = tmp;
	else	
		p->next = tmp;
	
	return plist;
	
}

Priority *del_priority(Priority *plist, char *pkey) {
	
	Priority *p, *canc;
	
	if(!plist) return NULL;
	
	p = plist;
	
	if(strcmp(p->key, pkey) == 0) {
		canc = p;
		p = p->next;
		free(canc);
		return p;
	}
	
	while(p->next != NULL && strcmp(p->next->key, pkey) != 0) p = p->next;
	
	if(p->next) {
		canc = p->next;
		p = p->next->next;
		free(canc);
	}
	
	return p;
	
}	

unsigned char *gennewkey(int keylen, int keyblock, unsigned char *cset, int reset) {
	
	static unsigned char *pass = NULL;
	static mybrute_t *brute = NULL;
	char *pkey = NULL;
	
	if(pkey) free(pkey);
	
	/* Priority list managment */
	if(plist) {
		pkey = strdup(plist->key);
		if(!pkey) {
			perror("\ngennewkey priority key");
			goto noplist;
		}
		plist = del_priority(plist, pkey);
		return (unsigned char *)pkey;
	}
	
	/* Priority list only */
	if(plist_only) {
		plist_only = 0;
		c_session = 0;
	}
	
noplist:
	
	if(reset) {
		if(pass) free(pass);
		pass = NULL;
		brute = NULL;
		return NULL;
	}
	
	/* Session closed */
	if(!c_session) return NULL;
	
	if(!brute) {
		
		brute = mybrute_init(keylen-keyblock, cset);
		pass = calloc(keylen+1, sizeof(char));
		free(brute->buff);
		brute->buff = pass + keyblock;
		
		memset(pass, cset[0], keyblock);
		
		/* First key to crack */
		return (unsigned char *)"";
		
	}
	
	/* Bruteforce ended */
	if(!mybrute(brute, 0)) {
		c_session = 0;
		return NULL;
	}
		
	return (unsigned char *)strdup((char *)pass);
	
}

int string2uchar(unsigned char *in, unsigned char *out, size_t outlen) {

	int i, t;
	unsigned char *p;

	p = in;

	for(i=0; i<outlen; i++) {
		if(sscanf((char *)p, "%02x", &t) != 1) return -1;
		out[i] = t;
		p += 2;
	}

	return 0;

}

int get_len_by_id(int id) {

	switch(id) {

		/* MD5 */
		case 0:
			return 16;
		break;

		/* SHA-1 */
		case 1:
			return 20;
		break;

		default:
			return -1;
		break;
		
	}

	return -1;

}

int delimit_str(char *str) {
	
	char *p = NULL;

	for(p=str; *p && *p != '\n' && *p != '\r'; p++);

	*p = 0x00;

	return p-str;

}

int mystrcpy(char *out, char *in, size_t inlen) {

	int i;
	
	if(inlen <= 0) return -1;
	
	for(i=0, inlen--; i<inlen && in[i]; i++) out[i] = in[i];
	
	out[i] = 0;

	return i;

}

int test_null_hash() {
	
	int hlen;
	unsigned char hashin[MAX_HASH_LENGTH],
	              hashout[MAX_HASH_LENGTH];
	              
	pthread_mutex_lock(&mutex);
			
	hlen = get_len_by_id(info_data->hashid);

	string2uchar((unsigned char *)info_data->hash, hashin, sizeof(hashin));

	switch(info_data->hashid) {

		/* MD5 */
		case 0:
			md5((unsigned char *)"", 0, hashout);
		break;

		/* SHA-1 */
		case 1:
			sha1((unsigned char *)"", 0, hashout);
		break;
				
		default:
			memset(hashout, 0, sizeof(hashout));
		break;

	}

	/* Incorrect received password */
	if(!memcmp(hashout, hashin, hlen)) {
		mystrcpy(psw, "", info_data->keylen+1);
		mystrcpy(ip, "127.0.0.1", sizeof(ip));
		pswfound = 1;
		return 0;
	}
			
	pthread_mutex_unlock(&mutex);

	return -1;	              
	              
}
